package io.ghyeok.channel

import android.content.Context
import android.util.Log
import com.zoyi.channel.plugin.android.ChannelIO
import com.zoyi.channel.plugin.android.ChannelPluginSettings
import com.zoyi.channel.plugin.android.Profile
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import java.nio.channels.Channel

class MethodCallHandlerImpl(private val context: Context) : MethodChannel.MethodCallHandler {

    override fun onMethodCall(call: MethodCall, result: MethodChannel.Result) {
        when (call.method) {
            "boot" -> boot(call, result)
            "show" -> show(call, result)
            "hide" -> hide(call, result)
            "shutdown" -> shutdown(call, result)
            "open" -> open(call, result)
            else -> result.notImplemented()
        }
    }

    private fun boot(call: MethodCall, result: MethodChannel.Result) {

        // PluginKey 설정되어있지 않는경우
        if (!call.hasArgument("pluginKey")) {
            result.error("ARGUMENT_ERROR", "plugin key is not set", "플리그인 키가 설정되어있지 않습니다.")
            return
        }

        // Plugin Setting
        val pluginKey: String? = call.argument("pluginKey")
        val settings = ChannelPluginSettings(pluginKey).apply {
            memberId = call.argument("memberId") as? String
            memberHash = call.argument("memberHash") as? String
        }

        // Profile Info
        val profileArgument: Map<String, String>? = call.argument("profile")
        val profile: Profile? = profileArgument?.run {
            val profile = Profile.create()

            val defaultProperties = arrayOf("name", "email", "avatarUrl", "mobileNumber")

            // 기본 정보를 제외한 다른 정보는 Properties 처리
            this.filterKeys { !defaultProperties.contains(it) }
                    .entries
                    .forEach { profile.setProperty(it.key, it.value) }

            // Set Default Property
            profile.name = get("name")
            profile.email = get("email")
            profile.avatarUrl = get("avatarUrl")
            profile.mobileNumber = get("mobileNumber")

            return@run profile
        }

        Log.d("ChannelIO",profile?.toJson()?:"NULL profile")

        // Boot ChannelIO
        ChannelIO.boot(settings, profile)

        result.success(true)
    }

    private fun open(call: MethodCall, result: MethodChannel.Result) {
        ChannelIO.open(context)

        result.success(true)
    }

    private fun show(call: MethodCall, result: MethodChannel.Result) {
        ChannelIO.show()

        result.success(true)
    }

    private fun hide(call: MethodCall, result: MethodChannel.Result) {
        ChannelIO.hide()

        result.success(true)
    }

    private fun shutdown(call: MethodCall, result: MethodChannel.Result) {
        ChannelIO.shutdown()

        result.success(true)
    }
}